<?php

namespace Drupal\alternative_user_emails\Plugin\Validation\Constraint;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\user\UserStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Validates that alternative_emails_field emails are unique on the site.
 */
class AlternativeUserEmailsValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * User storage handler.
   *
   * @var \Drupal\user\UserStorageInterface
   */
  protected $userStorage;

  /**
   * Constructs a new AlternativeUserEmailsValidator.
   *
   * @param \Drupal\user\UserStorageInterface $user_storage
   *   The user storage handler.
   */
  public function __construct(UserStorageInterface $user_storage) {
    $this->userStorage = $user_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('entity_type.manager')->getStorage('user'));
  }

  /**
   * {@inheritdoc}
   */
  public function validate($items, Constraint $constraint) {
    if (!$item = $items->first()) {
      return;
    }

    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $items->getEntity();
    if ($entity->getEntityTypeId() !== 'user' || $items->getFieldDefinition()->getName() !== 'alternative_user_emails') {
      return;
    }

    $id_key = $entity->getEntityType()->getKey('id');
    $entity_id = $entity->id();

    // Validate each alternative email given.
    foreach ($items as $item) {
      // Query for other users with the same email.
      $query = $this->userStorage->getQuery();
      $query->accessCheck(FALSE);

      // Exclude the current user.
      // Using isset() instead of !empty() as 0 and '0' are valid ID values for
      // entity types using string IDs.
      if (isset($entity_id)) {
        $query->condition($id_key, $entity_id, '<>');
      }

      // Check the primary and alternative emails on other users.
      $or = $query->orConditionGroup()
        ->condition('mail', $item->value)
        ->condition('alternative_user_emails', $item->value);
      $query->condition($or);

      $value_taken = (bool) $query
        ->range(0, 1)
        ->count()
        ->execute();

      if ($value_taken) {
        $this->context->addViolation($constraint->message, [
          '%value' => $item->value,
        ]);
      }
    }

  }

}
